#!/usr/bin/env python3
# Written in December 2023 by
# Pier Angelo Vendrame <vogliadifarniente@gmail.com>.
# To the extent possible under law, the author has dedicated all
# copyright and related and neighboring rights to this software to the
# public domain worldwide.
# THIS SOFTWARE IS DISTRIBUTED WITHOUT ANY WARRANTY.
import gzip
from pathlib import Path

# Let someone else handle type conversions :)
import imapclient


# With Python 3.12+ you can use itertools.batched
def batches(l, n):
    rv = []
    for i in range(0, len(l), n):
        up_to = min(len(l), i + n)
        rv.append(l[i:up_to])
    return rv


def process(username, password, directory, host):
    print(f"Processing {username}")
    directory = Path(directory)
    directory.mkdir(parents=True, exist_ok=True)
    with imapclient.IMAPClient(host=host) as client:
        client.login(username, password)
        for fol in client.list_folders():
            if fol[2] == "INBOX.Trash" or fol[2] == "INBOX.SPAM":
                continue
            (directory / fol[2]).mkdir(parents=True, exist_ok=True)
            client.select_folder(fol[2], readonly=True)
            messages = client.search("ALL")
            # Split in batches because fetch will download all the
            # messages to memory before returning.
            for mess in batches(messages, 100):
                response = client.fetch(mess, ["RFC822"])
                for message_id, data in response.items():
                    # If you want to be able to re-run the script after
                    # some time, you should use a smarter strategy for
                    # file names.
                    dest = directory / fol[2] / f"{message_id}.eml.gz"
                    if dest.exists():
                        print(f"Nope, {dest} already exists!")
                        continue
                    with gzip.open(dest, "wb") as gz:
                        gz.write(data[b"RFC822"])
    print(f"Done {username}")


# Important: use a Path
process(
    "username",
    "password",
    Path("destination-path"),
    "host"
)
